/*
 *  Hand Gesture Recognition Module
 * 
 *  Copyright (c) 2010-2012
 *  Nintendo Co., Ltd.
 *  All rights reserved.
 *
 * - API definition header file
 *
 */

#ifndef MW_HGR_HGR_H_
#define MW_HGR_HGR_H_

#include <nn/types.h>

/*------------------------------------------------------------------------------------*/
#define		HGRCG_FINMAX		(   10 )			/* Maximum number of recognized fingers				  */
#define		HGRCG_OBJMAX		(   32 )			/* Maximum number of recognized objects			  */
/*------------------------------------------------------------------------------------*/
#define		HGPAR_MODEMIN		(    0 )			/* Minimum value for the movement recognition mode			  */
#define		HGPAR_MODEMAX		(    2 )			/* Maximum value for the movement recognition mode			  */
/*------------------------------------------------------------------------------------*/
#define		HGPAR_NUMBMIN		(    1 )			/* Minimum value for specifying number of fingers				  */
#define		HGPAR_NUMBMAX		(   10 )			/* Maximum value for specifying number of fingers				  */
/*------------------------------------------------------------------------------------*/
#define		HGPAR_ANGLMIN		(    0 )			/* Minimum value of specifying range of finger angles			  */
#define		HGPAR_ANGLMAX		(    1 )			/* Maximum value of specifying range of finger angles			  */
/*------------------------------------------------------------------------------------*/
#define		HGPAR_LENGMIN		(   50 )			/* Minimum value for specifying minimum value of finger length		  */
#define		HGPAR_LENGMAX		(  350 )			/* Maximum value for specifying minimum value of finger length		  */
/*------------------------------------------------------------------------------------*/
#define		HGIMG_WIDMIN		(  320 )			/* Minimum value for original image width				  */
#define		HGIMG_WIDMAX		(  640 )			/* Maximum value for original image width				  */
#define		HGIMG_HEIMIN		(  240 )			/* Minimum value for original image height				  */
#define		HGIMG_HEIMAX		(  480 )			/* Maximum value for original image height				  */
/*------------------------------------------------------------------------------------*/
#define		HGEXT_RECTMIN		(  200 )			/* Minimum value for width and height of the rectangular region targeted for processing */
/*------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------*/
#define		PSERR_TRUE			(    0 )			/* Processing successful				  */
#define		PSERR_MEMR			(   -1 )			/* Insufficient memory					  */
#define		PSERR_ARGU			(   -2 )			/* Argument error						  */
#define		PSERR_IMAG			(   -3 )			/* Image abnormality					  */
#define		PSERR_PROC			(   -4 )			/* Processing not impossible			  */
#define		PSERR_RECG			(   -5 )			/* Recognition failure					  */
/*------------------------------------------------------------------------------------*/

namespace mw { namespace hgr {

typedef struct {

	u8			Z ;								/* Dummy						  */
    u8			B ;								/* Image Value Green Component 				  */
    u8			G ;								/* Image Value Red Component 				  */
    u8			R ;								/* Image Value Blue Component    				  */

} RGBColor;

typedef struct {

    s32			xSize  ;						/* Image data width				  */
    s32			ySize  ;						/* Image data height				  */
    RGBColor	*pcImg ;						/* Color image data				  */

} PSCimg ;

typedef struct {

	u8			Y0 ;							/* Y( n ) component					  */
    u8			U  ;							/* U( n ) component 					  */
    u8			Y1 ;							/* Y(n+1) component					  */
    u8			V  ;							/* V( n ) component	   				  */

} YUVColor;

typedef struct {

    u32			xSize  ;						/* Image data width				  */
    u32			ySize  ;						/* Image data height				  */
    YUVColor	*pcImg ;						/* Color image data				  */

} PSYUVimg ;

typedef struct {

    s32			x ;								/* Point x coordinate [pixel]				  */
    s32			y ;								/* Point y coordinate [pixel]				  */

} PSPoint ;

typedef struct {

	s32			sx ;							/* Rectangular region start point upper left x coordinate [pixel]  */
	s32			sy ;							/* Rectangular region start point upper left y coordinate [pixel]  */
	s32			ex ;							/* Rectangular region end point lower right x coordinate [pixel]  */
	s32			ey ;							/* Rectangular region end point lower right y coordinate [pixel]  */

} PSRect ;

typedef struct {

    u32         fingernum              ;		/* Number of fingers						  */
    PSPoint		fpoint[HGRCG_FINMAX]   ;		/* Fingertip position					  */
	u16			fdir[HGRCG_FINMAX]     ;		/* Finger angle (0 through 359 degrees)				  */
	s32			fleng[HGRCG_FINMAX]    ;		/* Finger length						  */
	s32         reserve0               ;		/* Reserved 0						  */
	s32			reserve1               ;		/* Reserved 1						  */
	s32			reserve2               ;		/* Reserved 2						  */
	s32			reserve3               ;		/* Reserved 3						  */

} HGResult ;

typedef struct {

	s32			xsize    ;						/* Image data width (320,640)		  */
	s32			ysize    ;						/* Image data height (240,480)		  */
    PSRect		hgrect   ; 						/* Rectangular region targeted for processing				  */
    s16         hgmode   ;						/* Recognition movement mode (0,1)			  */
	s16			hgnumb   ;						/* The number of fingers detected (1 to 10)			  */
	s16			hganglflg;						/* Finger angle range (0: 180 degrees; 1: 360 degrees) */
	s16			hgminleng;						/* Finger length minimum value specification (50 through 350)	  */
    s16			hgmaxleng;						/* Finger length maximum value specification (50 through 350)	  */
    u8			reserve0 ;						/* Reserve Flag 0					  */
    u8			reserve1 ;						/* Reserve Flag 1					  */

} HGConfig ;

s32 		HGRcgGetWorkSize		( HGConfig*, u32*     ) ;
s32 		HGRcgProcessInit		( HGConfig*, u8*     ) ;
s32 		HGRcgProcessExec		( HGConfig*, PSYUVimg*, HGResult* ) ;


}}

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {

    s32			x ;								/* Point x coordinate [pixel]				  */
    s32			y ;								/* Point y coordinate [pixel]				  */

} mwhgrPSPoint ;

typedef struct {

	s32			sx ;							/* Rectangular region start point upper left x coordinate [pixel]  */
	s32			sy ;							/* Rectangular region start point upper left y coordinate [pixel]  */
	s32			ex ;							/* Rectangular region end point lower right x coordinate [pixel]  */
	s32			ey ;							/* Rectangular region end point lower right y coordinate [pixel]  */

} mwhgrPSRect ;

typedef struct {

    u8			B ;								/* Image Value Green Component 				  */
    u8			G ;								/* Image Value Red Component 				  */
    u8			R ;								/* Image Value Blue Component    				  */
	u8			Z ;								/* Dummy						  */

} mwhgrRGBColor;

typedef struct {

    s32				xSize  ;					/* Image data width				  */
    s32				ySize  ;					/* Image data height				  */
    mwhgrRGBColor	*pcImg ;					/* Color image data				  */

} mwhgrPSCimg ;

typedef struct {

	u8			Y0 ;							/* Y( n ) component					  */
    u8			U  ;							/* U( n ) component 					  */
    u8			Y1 ;							/* Y(n+1) component					  */
    u8			V  ;							/* V( n ) component	   				  */

} mwhgrYUVColor;

typedef struct {

    u32				xSize  ;					/* Image data width				  */
    u32				ySize  ;					/* Image data height				  */
    mwhgrYUVColor	*pcImg ;					/* Color image data				  */

} mwhgrPSYUVimg ;


typedef struct {

    u32				bodynum                ;	/* Number of moving bodies						  */
    mwhgrPSPoint	fpoint[HGRCG_FINMAX]   ;	/* Fingertip position					  */
	u16				fdir[HGRCG_FINMAX]     ;	/* Finger angle (0 through 359 degrees)				  */
	s32				fleng[HGRCG_FINMAX]    ;	/* Finger length						  */
	s32				reserve0               ;	/* Reserved 0						  */
	s32				reserve1               ;	/* Reserved 1						  */
	s32				reserve2               ;	/* Reserved 2						  */
	s32				reserve3               ;	/* Reserved 3						  */

} mwhgrHGResult ;

typedef struct {

	s32				xsize    ;					/* Image data width (320,640)		  */
	s32				ysize    ;					/* Image data height (240,480)		  */
    mwhgrPSRect		hgrect   ; 					/* Rectangular region targeted for processing				  */
    s16				hgmode   ;					/* Recognition movement mode (0,1)			  */
	s16				hgnumb   ;					/* The number of fingers detected (1 to 10)			  */
	s16				hganglflg;					/* Finger angle range (0: 180 degrees; 1: 360 degrees) */
	s16				hgminleng;					/* Finger length minimum value specification (100 through 300)	  */
    s16				hgmaxleng;					/* Finger length maximum value specification (50 through 350)	  */
    u8				reserve0 ;					/* Reserve Flag 0					  */
    u8				reserve1 ;					/* Reserve Flag 1					  */

} mwhgrHGConfig ;

s32 	mwhgrHGRcgGetWorkSize		( mwhgrHGConfig*, u32*     ) ;
s32 	mwhgrHGRcgProcessInit		( mwhgrHGConfig*, u8*     ) ;
s32		mwhgrHGRcgProcessExec		( mwhgrHGConfig*, mwhgrPSYUVimg*, mwhgrHGResult* ) ;

#ifdef __cplusplus
}
#endif

#endif /* MW_HGR_HGR_H_ */

