// ----------------------------------------------------------------------
// Overview
//   Menu class
// ----------------------------------------------------------------------
#ifndef __TXT_MENU__
#define __TXT_MENU__

// ----------------------------------------------------------------------
// Item type
// ----------------------------------------------------------------------
typedef enum
{
	eITEM_ONOFF,
	eITEM_MINMAX,
	eITEM_SUB_MENU,
	eITEM_EXIT
}eItemType;

// ----------------------------------------------------------------------
// Settings function pointer
// ----------------------------------------------------------------------
typedef void	(*FUNC)		(int);
typedef void	(*FUNC_STR) (int, char*);

// ----------------------------------------------------------------------
// Menu information
// ----------------------------------------------------------------------
typedef struct
{
	int			id;
	eItemType	type;
	bool		bOnOff;
	char		padding[2];
	int			min;
	int			max;
	int			nDefault;
	char		str[64];
	void*		parent;
	void*		child;
	FUNC		func;
	FUNC_STR	getStr;
	int			add;		// 2010.12.01 Added additional volume

}ITEM_INFO, *LPITEM_INFO;

// ----------------------------------------------------------------------
// class
// ----------------------------------------------------------------------
class CTxtMenu
{
public:
	CTxtMenu(int maxItem, char* title);
	~CTxtMenu();

	// Added menu
	void		SetItemOnOff	(int no, int nID, char* str, bool bDefault, FUNC func);
	void		SetItemMinMax	(int no, int nID, char* str, int min, int max, int nDefault, FUNC func, int add = 1);
	void		SetSubMenu		(int no, int nID, char* str, CTxtMenu* menu, FUNC func);
	void		SetItemExit		(int no, int nID, char* str, FUNC func);

	void		SetItemStrFunc	(int nID, FUNC_STR getStr);


	// Get the menu information
	void		GetTitle		(char* str);
	int			GetCount		();
	void		GetItemText		(int no, char* str);

	bool		GetItemOnOff	(int nID);
	int			GetItemMinMax	(int nID);

	// 
	void		SetNext();
	void		SetPrev();
	void		SetSelect(bool bUp = false);

protected:
	void		SetParent(CTxtMenu* parent);
	CTxtMenu*	 SetChiledSelect(bool bUp);

	char		m_title[64];
	LPITEM_INFO	m_item_info;
	int			m_maxItem;
	int			m_currItem;
	int			m_depth;

	CTxtMenu*	m_menu;
	CTxtMenu*	m_parent;
};


#endif // __TXT_MENU__
