// ----------------------------------------------------------------------
// Main menu class
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/gx.h>
#include	<nn/math.h>
#include	<nn/os.h>
#include	<nn/hid.h>
#include	<nn/fnd.h>
#include	<nn/fs.h>

#include	<string.h>

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"CTR_Define.h"
#include	"CtrlGraphics.h"
#include	"CtrlCamera.h"
#include	"TxtWindow.h"
#include	"TxtWinMenu.h"

// ----------------------------------------------------------------------
// using namespace
// ----------------------------------------------------------------------
using namespace nn::hid;

typedef struct
{
	int hgmode;
	int hgnumb;
	int hganglflg;
	int hgminleng;
	int hgmaxleng;
}HGRLibParam, *LPHGRLibParam;

// ----------------------------------------------------------------------
// global
// ----------------------------------------------------------------------
static int g_quality	= 2;
static HGRLibParam g_HGRLibParam;

// ----------------------------------------------------------------------
// SetDrawQuality
// ----------------------------------------------------------------------
// Content
//   Changes render quality.
//   1: Render as is
//   2: Make height and width to 1/2, enlarge then render
//   3: Make height and width to 1/4, enlarge then render
//   Method 1 is higher quality, and method 3 is faster.
// ----------------------------------------------------------------------
// Arguments
//   n		[ I ] Specifies parameters after modification.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void SetDrawQuality(int n)
{
	switch(n)
	{
	case 1:g_quality = 1;break;
	case 2:g_quality = 2;break;
	case 3:g_quality = 4;break;
	}
}

// ----------------------------------------------------------------------
// CTxtWinMenu::GetDrawQuality
// ----------------------------------------------------------------------
// Content
//   Gets render quality.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   Returns quality value (1, 2, 4).
// ----------------------------------------------------------------------
int CTxtWinMenu::GetDrawQuality()
{
	return g_quality;
}

// ----------------------------------------------------------------------
// SetHgMode/SetHgNumb/SetHgAnglFlg/SetHgMinLeng
// ----------------------------------------------------------------------
// Content
//   Changes HGRLib parameter.
// ----------------------------------------------------------------------
// Arguments
//   n		[ I ] Specifies parameters after modification.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void SetHgMode   (int n) { g_HGRLibParam.hgmode    = n; }
void SetHgNumb   (int n) { g_HGRLibParam.hgnumb    = n; }
void SetHgAnglFlg(int n) { g_HGRLibParam.hganglflg = n; }
void SetHgMinLeng(int n) { g_HGRLibParam.hgminleng = n; }
void SetHgMaxLeng(int n) { g_HGRLibParam.hgmaxleng = n; }


// ----------------------------------------------------------------------
// CTxtWinMenu::GetHgMode/CTxtWinMenu::GetHgNumb/CTxtWinMenu::GetHgAnglFlg
// CTxtWinMenu::GetHgMinLeng/CTxtWinMenu::GetHgMaxLeng
// ----------------------------------------------------------------------
// Content
//   Gets HGRLib parameter values.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   Returns each value.
// ----------------------------------------------------------------------
int CTxtWinMenu::GetHgMode()	{ return g_HGRLibParam.hgmode;	 }
int CTxtWinMenu::GetHgNumb()	{ return g_HGRLibParam.hgnumb; }
int CTxtWinMenu::GetHgAnglFlg()	{ return g_HGRLibParam.hganglflg; }
int CTxtWinMenu::GetHgMinLeng()	{ return g_HGRLibParam.hgminleng; }
int CTxtWinMenu::GetHgMaxLeng()	{ return g_HGRLibParam.hgmaxleng; }

// ----------------------------------------------------------------------
// CTxtWinMenu::CTxtWinMenu
// ----------------------------------------------------------------------
// Content
//   Constructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWinMenu::CTxtWinMenu()
	: CTxtWindow("Menu"),
	m_menu(3, "PROC SIZE"),
	m_menu_sa(3, "SETTING APPLICATION"),
	m_menu_sh(6, "SETTING HGRLIB"),
	m_menu_sc(9, "SETTING GESTURE")
{
	// ----------------------------------------------------------------------
	// Application Menu
	// ----------------------------------------------------------------------
	m_menu_sa.SetItemOnOff	(0, eMENU_T_SA_ENABLE,				"Hand Gesture       ", true, NULL);
	m_menu_sa.SetItemMinMax	(1, eMENU_T_SA_DRAWQUALITY,			"Draw Quality       ",  1,  3, 2, SetDrawQuality);
	m_menu_sa.SetItemExit	(2, eMENU_T_SA_EXIT,				"Exit               ", NULL);

	// ----------------------------------------------------------------------
	// HGRLib Menu
	// ----------------------------------------------------------------------
	m_menu_sh.SetItemMinMax	(0, eMENU_T_SH_HGMODE,				"HG Mode            ",   0,   2,   1, SetHgMode);
	m_menu_sh.SetItemMinMax	(1, eMENU_T_SH_HGNUMB,				"Numb   Param       ",   1,  10,  10, SetHgNumb);
	m_menu_sh.SetItemMinMax	(2, eMENU_T_SH_HGANGLFLG,			"Angle  Param       ",   0,   1,   0, SetHgAnglFlg);
	m_menu_sh.SetItemMinMax	(3, eMENU_T_SH_HGMINLENG,			"MinLen Param       ",  50, 350,  50, SetHgMinLeng, 10);
	m_menu_sh.SetItemMinMax	(4, eMENU_T_SH_HGMAXLENG,			"MaxLen Param       ",  50, 350, 350, SetHgMaxLeng, 10);
	m_menu_sh.SetItemExit	(5, eMENU_T_SH_EXIT,				"Exit               ", NULL);

	// ----------------------------------------------------------------------
	// Camera menu
	// ----------------------------------------------------------------------
	m_menu_sc.SetItemMinMax	(0, eMENU_T_SC_EXPOSURE,			"Exposure           ", -5,  5,  0, SetExporse);
	m_menu_sc.SetItemMinMax	(1, eMENU_T_SC_WHITEBALANCE,		"White Balance      ",  0,  5,  2, SetWhiteBalance);
	m_menu_sc.SetItemMinMax	(2, eMENU_T_SC_SHARPNESS,			"Sharpness          ", -4,  5,  0, SetSharpness);
	m_menu_sc.SetItemOnOff	(3, eMENU_T_SC_AUTOEXPOSURE,		"Auto Exposure      ", false,	  SetAutoExporse);
	m_menu_sc.SetItemOnOff	(4, eMENU_T_SC_AUTOWHITEBALANCE,	"Auto White Balance ", false,	  SetAutoWhiteBalance);
	m_menu_sc.SetItemMinMax	(5, eMENU_T_SC_PHOTOMODE,			"Photo Mode         ",  0,  4,  0, SetPhotoMode);
	m_menu_sc.SetItemMinMax	(6, eMENU_T_SC_CONTRAST,			"Contrast           ",  0, 10,  4, SetContrast);
	m_menu_sc.SetItemMinMax	(7, eMENU_T_SC_LENSCORRECTION,		"Lenscorrection     ",  0,  2,  1, SetLensCorrection);
	m_menu_sc.SetItemExit	(8, eMENU_T_SC_EXIT,				"Exit               ", NULL);

	m_menu_sc.SetItemStrFunc(eMENU_T_SC_PHOTOMODE,		GetPhotoModeStr);
	m_menu_sc.SetItemStrFunc(eMENU_T_SC_WHITEBALANCE,	GetWhiteBalanceStr);
	m_menu_sc.SetItemStrFunc(eMENU_T_SC_CONTRAST,		GetContrastStr);
	m_menu_sc.SetItemStrFunc(eMENU_T_SC_LENSCORRECTION,	GetLensCorrectionStr);

	// ----------------------------------------------------------------------
	// Root Menu
	// ----------------------------------------------------------------------
	m_menu.SetSubMenu(0, eMENU_T_SETTING_APPLICATION,	"Setting Application ", &m_menu_sa, NULL);
	m_menu.SetSubMenu(1, eMENU_T_SETTING_HGRLIB,		"Setting HGRLib      ", &m_menu_sh, NULL);
	m_menu.SetSubMenu(2, eMENU_T_SETTING_CAMERA,		"Setting Camera      ", &m_menu_sc, NULL);

	g_HGRLibParam.hgmode	= 1;
	g_HGRLibParam.hgnumb	= 10;
	g_HGRLibParam.hganglflg	= 0;
	g_HGRLibParam.hgminleng	= 50;
	g_HGRLibParam.hgmaxleng	= 350;
}

// ----------------------------------------------------------------------
// CTxtWinMenu::~CTxtWinMenu
// ----------------------------------------------------------------------
// Content
//   Destructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWinMenu::~CTxtWinMenu()
{
}

// ----------------------------------------------------------------------
// CTxtWinMenu::Draw
// ----------------------------------------------------------------------
// Content
//   Displays Window.
// ----------------------------------------------------------------------
// Arguments
//   graphics		[ I ] Specify graphics class.
//   display		[ I ] Specify Window show display.
//   sx				[ I ] Specify Window display x-coordinate.
//   sy				[ I ] Specify Window display y-coordinate.
// ----------------------------------------------------------------------
// Return value
//   Always returns 0.
// ----------------------------------------------------------------------
int CTxtWinMenu::Draw(CCtrlGraphics* graphics, const s32 display, int sx, int sy)
{
	char	str[64];
	int		n		 = 0;
	int		menu_num = m_menu.GetCount();

	CTxtWindow::SetLines(menu_num);
	for (n = 0 ; n < menu_num ; n++)
	{
		m_menu.GetItemText(n, str);
		CTxtWindow::SetText(n, str, 0.0, 0.0, 0.0);
	}

	CTxtWindow::DrawWindow(graphics, display, sx, sy);

	return 0;
}

// ----------------------------------------------------------------------
// CTxtWinMenu::IsEnableHandGesture
// ----------------------------------------------------------------------
// Content
//   Gets whether hand gesture is on or off.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   Returns true when on; returns false when off.
// ----------------------------------------------------------------------
bool CTxtWinMenu::IsEnableHandGesture()
{
	return m_menu_sa.GetItemOnOff(eMENU_T_SA_ENABLE);
}

// ----------------------------------------------------------------------
// CTxtWinMenu::Ctrl
// ----------------------------------------------------------------------
// Content
//   Performs operations with PAD.
//   - UP button: moves to item above.
//   - DOWN button: moves to item below.
//   - A Button: Confirm or adds to value.
//   - B Button: Confirm or subtracts from value.
// ----------------------------------------------------------------------
// Arguments
//   padStatus			[ I ] Specifies pad status.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtWinMenu::Ctrl(PadStatus* padStatus)
{
	// ----------------------------------------------------------------------
	// Up Button
	// ----------------------------------------------------------------------
	if (padStatus->trigger & BUTTON_UP)
	{
		m_menu.SetPrev();
	}

	// ----------------------------------------------------------------------
	// Down Button
	// ----------------------------------------------------------------------
	if (padStatus->trigger & BUTTON_DOWN)
	{
		m_menu.SetNext();
	}

	// ----------------------------------------------------------------------
	// A Button Determination (plus)
	// ----------------------------------------------------------------------
	if (padStatus->trigger & BUTTON_A)
	{
		m_menu.SetSelect(true);
	}

	// ----------------------------------------------------------------------
	// B Button Determination (minus)
	// ----------------------------------------------------------------------
	if (padStatus->trigger & BUTTON_B)
	{
		m_menu.SetSelect(false);
	}
}
