// ----------------------------------------------------------------------
// Detection results display window class
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/gx.h>
#include	<nn/math.h>
#include	<nn/os.h>
#include	<nn/hid.h>
#include	<nn/fnd.h>
#include	<nn/fs.h>

#include	<string.h>

// ----------------------------------------------------------------------
// include hgr header
// ----------------------------------------------------------------------
#include	"hgr.h"					// hgr header
using namespace mw::hgr;

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"CTR_Define.h"
#include	"CtrlGraphics.h"
#include	"CtrlCamera.h"
#include	"TxtWindow.h"
#include	"TxtWinResultHgr.h"

// ----------------------------------------------------------------------
// CTxtWinResultHgr::CTxtWinResultHgr
// ----------------------------------------------------------------------
// Content
//   Constructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWinResultHgr::CTxtWinResultHgr()
	: CTxtWindow("Result")
{

}

// ----------------------------------------------------------------------
// CTxtWinResultHgr::~CTxtWinResultHgr
// ----------------------------------------------------------------------
// Content
//   Destructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWinResultHgr::~CTxtWinResultHgr()
{
}

// ----------------------------------------------------------------------
// CTxtWinResultHgr::Draw
// ----------------------------------------------------------------------
// Content
//   Displays window.
// ----------------------------------------------------------------------
// Arguments
//   graphics		[ I ] Specify graphics class.
//   display		[ I ] Specify Window show display.
//   sx				[ I ] Specify Window display x-coordinate.
//   sy				[ I ] Specify Window display y-coordinate.
// ----------------------------------------------------------------------
// Return value
//   Always returns 0.
// ----------------------------------------------------------------------
int CTxtWinResultHgr::Draw(CCtrlGraphics* graphics, const s32 display, int sx, int sy)
{
	char		str[64];
	int			n			= 0;
	int			finger_num	= m_result.fingernum;
	PSPoint*	fpoint		= m_result.fpoint;
	unsigned short* fdir	= m_result.fdir;
	s32*		fleng		= m_result.fleng;
	int			lines		= 12;

	if (finger_num + 2 > lines)
	{
		lines = finger_num + 2; 
	}

	CTxtWindow::SetLines(lines);
	
	sprintf(str, "feanum:%d\n", finger_num);
	CTxtWindow::SetText(0, str, 0.0, 0.0, 0.0);

	sprintf(str, "------------------------------\n");
	CTxtWindow::SetText(1, str, 1.0, 1.0, 1.0);

	for (n = 0 ; n < finger_num ; n++)
	{
		sprintf(str,"[%3d] (%3d, %3d)  DIR = %3d Len = %3d\n", n, (int)fpoint->x, (int)fpoint->y ,(int)*fdir, (int)*fleng);
		CTxtWindow::SetText(n + 2, str, 0.0, 0.0, 0.0);

		fpoint++;
		fdir++;
		fleng++;
	}

	for (; n < 10 ; n++)
	{
		sprintf(str,"[%3d] (---, ---)  DIR = --- Len = ---\n", n);
		CTxtWindow::SetText(n + 2, str, 0.0, 0.0, 0.0);
	}

	CTxtWindow::DrawWindow(graphics, display, sx, sy);

	return 0;
}

// ----------------------------------------------------------------------
// CTxtWinResultHgr::UpdateResult
// ----------------------------------------------------------------------
// Content
//   Updates result value.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtWinResultHgr::UpdateResult(HGResult* result)
{
	memcpy(&m_result, result, sizeof(HGResult));
}
