// ----------------------------------------------------------------------
// Start up window class
// Displays at startup, and selects mode
//  - VGA (640 * 480) /QVGA (320 * 240)
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/gx.h>
#include	<nn/math.h>
#include	<nn/os.h>
#include	<nn/hid.h>
#include	<nn/fnd.h>
#include	<nn/fs.h>

#include	<string.h>

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"CTR_Define.h"
#include	"CtrlGraphics.h"
#include	"TxtWindow.h"
#include	"TxtWinStartup.h"

// ----------------------------------------------------------------------
// using namespace
// ----------------------------------------------------------------------
using namespace nn::hid;

// ----------------------------------------------------------------------
// Enums
// ----------------------------------------------------------------------
typedef enum
{
	eMENU_SUW_SIZE,
	eMENU_SUW_EXIT
}eMENU_SUW;

// ----------------------------------------------------------------------
// global 
// ----------------------------------------------------------------------
bool g_endflg = false;

// ----------------------------------------------------------------------
// func_end
// ----------------------------------------------------------------------
// Content
//   Called when Exit is selected.
//   Enable global DoModal end flag.
// ----------------------------------------------------------------------
// Arguments
//   n			[ I ] Not used.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void func_end(int n)
{
	g_endflg = (n != false) ? true : true;
}

// ----------------------------------------------------------------------
// GetSizeStr
// ----------------------------------------------------------------------
// Content
//   Gets string for displaying size.
// ----------------------------------------------------------------------
// Arguments
//   n			[ I ] Specifies value between 0 and 1.
//   str		[ O ] Stores acquired string.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void GetSizeStr(int n, char* str)
{
	switch(n)
	{
	case 0:	sprintf(str, "VGA  ");	break;
	case 1:	sprintf(str, "QVGA ");	break;
	}
}


// ----------------------------------------------------------------------
// CTxtWinStartup::CTxtWinStartup
// ----------------------------------------------------------------------
// Content
//   Constructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWinStartup::CTxtWinStartup()
	: CTxtWindow("Start up"),
	m_menu(2, "PROC SIZE")
{
	m_menu.SetItemMinMax(0, eMENU_SUW_SIZE, "VGA/QVGA  :", 0, 1, 1, NULL);
	m_menu.SetItemExit  (1, eMENU_SUW_EXIT, "Start      ", func_end);

	m_menu.SetItemStrFunc(eMENU_SUW_SIZE, GetSizeStr);

	CTxtWindow::SetForcus(true);
}

// ----------------------------------------------------------------------
// CTxtWinStartup::~CTxtWinStartup
// ----------------------------------------------------------------------
// Content
//   Destructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWinStartup::~CTxtWinStartup()
{

}

// ----------------------------------------------------------------------
// CTxtWinStartup::DoModal
// ----------------------------------------------------------------------
// Content
//   Displays window.
// ----------------------------------------------------------------------
// Arguments
//   graphics		[ I ] Specify graphics class.
//   display		[ I ] Specify Window show display.
//   sx				[ I ] Specify Window display x-coordinate.
//   sy				[ I ] Specify Window display y-coordinate.
// ----------------------------------------------------------------------
// Return value
//   Always returns 0.
// ----------------------------------------------------------------------
int CTxtWinStartup::DoModal(CCtrlGraphics* graphics, const s32 display, int sx, int sy)
{
	PadReader padReader;
	PadStatus padStatus;
	char str[64];
	int n = 0;

	while(g_endflg == false)
	{
		int menu_num = m_menu.GetCount();

		CTxtWindow::SetLines(menu_num);
		for (n = 0 ; n < menu_num ; n++)
		{
			m_menu.GetItemText(n, str);
			CTxtWindow::SetText(n, str, 1.0, 1.0, 1.0);
		}

		// ----------------------------------------------------------------------
		// Operate menu using controller +Control Pad
		// ----------------------------------------------------------------------
		padReader.ReadLatest(&padStatus);

		// UP Button
		if (padStatus.trigger & BUTTON_UP)
		{
			m_menu.SetPrev();
		}

		// DOWN Button
		if (padStatus.trigger & BUTTON_DOWN)
		{
			m_menu.SetNext();
		}

		// A Button Determination (plus)
		if (padStatus.trigger & BUTTON_A)
		{
			m_menu.SetSelect(true);
		}

		// B Button Determination (minus)
		if (padStatus.trigger & BUTTON_B)
		{
			m_menu.SetSelect(false);
		}

		graphics->ClearDisplay(display);

		CTxtWindow::DrawWindow(graphics, display, sx, sy);

		graphics->SwapBuffer(display);
	}
	return 0;
}

// ----------------------------------------------------------------------
// CTxtWinStartup::GetStartMode
// ----------------------------------------------------------------------
// Content
//   Returns which of the VGA/QVGA modes to use at startup.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   eSTART_VGA/eSTART_QVGA
// ----------------------------------------------------------------------
eSTART_MODE CTxtWinStartup::GetStartMode()
{
	switch(m_menu.GetItemMinMax(eMENU_SUW_SIZE))
	{
	case 0: return eSTART_VGA;
	case 1: return eSTART_QVGA;
	}

	return eSTART_QVGA;
}

