//**************************************************************************************
//
//		Hand Gesture Recognition Library (HGRLib)
//
//		HGRLib Native Source File
//
//		Copyright   :: (C) PANASONIC CORPORATION, SYSTEM NETWORKS COMPANY 2010
//		Date        :: 2010.03.23
//		CPU         :: **
//		Language    :: C,C++
//		File Name   :: demo.cpp
//
//**************************************************************************************

// ----------------------------------------------------------------------
// define
// ----------------------------------------------------------------------
#define		CPLUSPLUS

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/os.h>
#include	<nn/fs.h>
#include	<nn/fnd.h>
#include	<nn/hid.h>
#include	<nn/math.h>
#include	<nn/hid.h>
#include	<nn/camera.h>
#include	<nn/y2r.h>
#include	<nn/hid.h>
#include	<nn/applet.h>

// ----------------------------------------------------------------------
// include hgr header
// ----------------------------------------------------------------------
#include	"hgr.h"					// hgr header
using namespace mw::hgr;

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"CTR_Define.h"
#include	"CtrlGraphics.h"
#include	"CtrlCamera.h"
#include	"TxtMenu.h"
#include	"TxtWindow.h"
#include	"TxtWinStartup.h"
#include	"TxtWinMenu.h"
#include	"TxtWinResultHgr.h"
#include	"TxtWinDebugTimer.h"
#include	"DebugWriterHgr.h"
#include	"appdefine.h"

// ----------------------------------------------------------------------
// using namespace
// ----------------------------------------------------------------------
using namespace nn::hid;
using namespace nn::os;
using namespace nn::math;

// ----------------------------------------------------------------------
// global
// ----------------------------------------------------------------------
extern nn::fnd::ExpHeap gAppHeap;
static u8 *rgbBuffer;
static u8 *yuvBuffer;

// ----------------------------------------------------------------------
// nnMain
// ----------------------------------------------------------------------
void nnMain(void)
{
	unsigned char	*extBuf			= NULL;
	u32				workSize		= 0;
	HGConfig		hgConfig;

	PSYUVimg		hgYUVimg;

	HGResult		hgResult;
	int				fingernum		= 0;
	PSPoint			*fpoint			= NULL;
	unsigned short	*fdir			= NULL;

	int 			n				= 0;
	int				i				= 0;
	int				ret				= 0;

	int				proc_width		= 320;				// Overall process width: image input through rendering (excluding recognition)
	int				proc_height		= 240;				// Overall process height: image input through rendering (excluding recognition)
	Size			proc_size		= SIZE_QVGA;
	FrameRate		proc_rate		= FRAME_RATE_30;

	int				view_width		= nn::gx::CTR::DISPLAY1_HEIGHT;		// Width of View to display the camera image   (DISPLAY1_HEIGHT = 320) 
	int				view_height		= nn::gx::CTR::DISPLAY1_WIDTH;		// Height of View to display the camera image   (DISPLAY1_WIDTH = 240)

	int				hgr_width		= 160;				// Recognition processing image width
	int				hgr_height		= 120;				// Recognition processing image height

	int				txtWinSort		= 0;

	CCtrlGraphics	graphics;					// Graphics control class
	CCtrlCamera		camera;						// Camera control class

	// Window class
	CTxtWinStartup			startWnd;			// Start up Window
	CTxtWinMenu				menuWnd;			// Menu Window
	CTxtWinResultHgr		resultWnd;			// Result Window
	CTxtWinDebugTimer		dbgTimerWnd(8);		// Debug timer display window

	CDebugWriterHgr	dbgWriter;			// Debugging output.
	bool			dbg_init		= false;	

	char str[64];

	// ======================================================================

	PRINT_OUT("Demo Start\n");

    nn::Result result;

    // fs initialization
    nn::fs::Initialize();

    // Initialize hid
    result = nn::hid::Initialize();
    NN_UTIL_PANIC_IF_FAILED(result);

    // Call only the nn::applet::Enable function to also allow execution from the HOME Menu
    // HOME Menu transitions, POWER Button presses, and sleep are all unsupported
    nn::applet::Enable();

	// Prepare pad
	PadReader		padReader;
	PadStatus		padStatus;

	OS_MEM_INIT();
	FILE_IO_INIT();
	OS_TICK_INIT();

 	// ----------------------------------------------------------------------
	// Initialize graphics class
	// ----------------------------------------------------------------------
	graphics.InitializeGraphics();

 	// ----------------------------------------------------------------------
	// Displays the Start up Window, and either VGA mode or QVGA mode is selected
	// ----------------------------------------------------------------------
	startWnd.DoModal(&graphics, NN_GX_DISPLAY1, -1, -1);
	
	eSTART_MODE eStartMode = startWnd.GetStartMode();

	if (eStartMode == eSTART_VGA)
	{
		proc_width		= 640;				// Input image/Render image width
		proc_height		= 480;				// Input image/Render image height
		proc_size		= SIZE_VGA;			// Input size
		proc_rate		= FRAME_RATE_20;	// Input frame rate
		hgr_width		= 640;				// Recognition processing image width
		hgr_height		= 480;				// Recognition processing image height
	}
	else if (eStartMode == eSTART_QVGA)
	{
		proc_width		= 320;				// Input image/Render image width
		proc_height		= 240;				// Input image/Render image height
		proc_size		= SIZE_QVGA;		// Input size
		proc_rate		= FRAME_RATE_30;	// Input frame rate
		hgr_width		= 320;				// Recognition processing image width
		hgr_height		= 240;				// Recognition processing image height
	}

 	// ----------------------------------------------------------------------
	// Initialize camera class
 	// ----------------------------------------------------------------------
	camera.InitializeCamera(SELECT_IN1, proc_size, proc_rate);

	dbgTimerWnd.SetTimerText(0, "recv camera    ");
	dbgTimerWnd.SetTimerText(1, "create cimg    ");
	dbgTimerWnd.SetTimerText(2, "proc exec      ");
	dbgTimerWnd.SetTimerText(3, "draw disp0     ");
	dbgTimerWnd.SetTimerText(4, "create texture ");
	dbgTimerWnd.SetTimerText(5, "draw disp1     ");
	dbgTimerWnd.SetTimerText(6, "other          ");
	dbgTimerWnd.SetTimerText(7, "swap buffer    ");

	// ----------------------------------------------------------------------
	// Settings
	// ----------------------------------------------------------------------
	hgConfig.xsize		= hgr_width;				// Image data width (320,640)
	hgConfig.ysize		= hgr_height;				// Image data height ( 240,480)
	hgConfig.hgrect.sx	= 0;						// Rectangular region targeted for processing
	hgConfig.hgrect.sy	= 0;						// Rectangular region targeted for processing
	hgConfig.hgrect.ex	= hgr_width - 1;			// Rectangular region targeted for processing
	hgConfig.hgrect.ey	= hgr_height - 1;			// Rectangular region targeted for processing
	hgConfig.hgmode		= 1;						// Recognition movement mode		(0, 1)
	hgConfig.hgnumb		= 10;						// The number of fingers detected (1 to 10)
	hgConfig.hganglflg	= 0;						// Finger angle range (0: 180 degrees; 1: 360 degrees)
	hgConfig.hgminleng	= 50;						// Finger length minimum value specification (50 through 350)
	hgConfig.hgmaxleng	= 350;						// Finger length maximum value specification (50 through 350)
	hgConfig.reserve0	= 0;						// Reserve Flag 0
	hgConfig.reserve1	= 0;						// Reserve Flag 1

	// ----------------------------------------------------------------------
	// Library function: Get size of required working memory
	// ----------------------------------------------------------------------
	ret = HGRcgGetWorkSize(&hgConfig, &workSize);
	if (ret != PSERR_TRUE)
	{
		PRINT_OUT("HGRcgGetWorkSize NG\n");
		return;
	}
	PRINT_OUT("HGRcgGetWorkSize OK[%ld]\n", workSize);

	extBuf = (u8*)OS_MALLOC(workSize);
	if (extBuf == NULL)
	{
		PRINT_OUT("malloc extBuf NG\n");
		return;
	}

    rgbBuffer = (u8*)OS_MALLOC_EX( 640 * 480 * 4, 64 );
    yuvBuffer = (u8*)OS_MALLOC_EX( 640 * 480 * 2, 64 );

	hgYUVimg.xSize = proc_width;
	hgYUVimg.ySize = proc_height;
	hgYUVimg.pcImg = (YUVColor*)yuvBuffer;

	// ----------------------------------------------------------------------
	// Library function: Initialization
	// ----------------------------------------------------------------------
	ret = HGRcgProcessInit(&hgConfig, extBuf);
	if (ret != PSERR_TRUE)
	{
		PRINT_OUT("HGRcgProcessInit NG\n");
		OS_FREE(extBuf);
		return;
	}
	PRINT_OUT("HGRcgProcessInit OK\n");

	// ----------------------------------------------------------------------
	// Settings for each Window
	// ----------------------------------------------------------------------
	menuWnd.SetWindowWidth(nn::gx::DISPLAY0_HEIGHT);

	resultWnd.SetWindowWidth(306);
	resultWnd.SetWindowHeight(120);

	menuWnd.SetForcus(true);

	// ----------------------------------------------------------------------
	// Main loop.
	// ----------------------------------------------------------------------
	int frame	 = 0;
	while(1)
	{
		dbgTimerWnd.TimerAllStart();

		// ----------------------------------------------------------------------
		// Get camera data
		// ----------------------------------------------------------------------
		camera.Receive(rgbBuffer,yuvBuffer, dbgTimerWnd);

		// ----------------------------------------------------------------------
		// Create PSCimg (since the top is (0, 0), reverse top and bottom) 
		// 2011.06.16 Deleted because became unnecessary with YUV support
		// ----------------------------------------------------------------------
		dbgTimerWnd.TimerStart(1);
		if (menuWnd.IsEnableHandGesture() == true)
		{
		}
		dbgTimerWnd.TimerEnd(1);

		// ----------------------------------------------------------------------
		// Recognition
		// ----------------------------------------------------------------------
		hgResult.fingernum	= 0;
		for (i = 0 ; i < HGRCG_FINMAX ; i++)
		{
			hgResult.fpoint[i].x = 0;
			hgResult.fpoint[i].y = 0;
			hgResult.fdir[i] = 0;
			hgResult.fleng[i] = 0;
		}
		hgResult.reserve0	= 0;
		hgResult.reserve1	= 0;
		hgResult.reserve2	= 0;
		hgResult.reserve3	= 0;

		hgConfig.hgmode		= menuWnd.GetHgMode();
		hgConfig.hgnumb		= menuWnd.GetHgNumb();
		hgConfig.hganglflg	= menuWnd.GetHgAnglFlg();
		hgConfig.hgminleng	= menuWnd.GetHgMinLeng();
		hgConfig.hgmaxleng	= menuWnd.GetHgMaxLeng();
		dbgTimerWnd.TimerStart(2);
		if (menuWnd.IsEnableHandGesture() == true)
		{
			// ----------------------------------------------------------------------
			// Library function: execute recognition
			// ----------------------------------------------------------------------
			ret = HGRcgProcessExec(&hgConfig, &hgYUVimg, &hgResult);
		}
		dbgTimerWnd.TimerEnd(2);

		fingernum	= hgResult.fingernum;
		fpoint		= hgResult.fpoint;

		// ======================================================================
		// Upper screen.
		// ======================================================================
		dbgTimerWnd.TimerStart(3);
		graphics.ClearDisplay(NN_GX_DISPLAY0);

		// ----------------------------------------------------------------------
		// Set the recognition result
		// ----------------------------------------------------------------------
		resultWnd.UpdateResult(&hgResult);

		// ----------------------------------------------------------------------
		// Render Window
		// ----------------------------------------------------------------------
		for (n = txtWinSort ; n < txtWinSort + 3 ; n++)
		{
			switch((n + 1) % 3)
			{
			case 0: menuWnd.Draw	(&graphics, NN_GX_DISPLAY0,   0,  0);	break;
			case 1: resultWnd.Draw	(&graphics, NN_GX_DISPLAY0,  40, 80);	break;
			case 2:	dbgTimerWnd.Draw(&graphics, NN_GX_DISPLAY0,  10, 60);	break;
			}
		}

		// ----------------------------------------------------------------------
		// Render frame in bottom right
		// ----------------------------------------------------------------------
		sprintf(str, "frame:[%08d]\n", frame++);
		graphics.SetColor(NN_GX_DISPLAY0, 0.0, 1.0, 0.0);
		graphics.DrawTextEx(NN_GX_DISPLAY0, 272, 214, str);

		// ----------------------------------------------------------------------
		// Swap
		// ----------------------------------------------------------------------
		graphics.SwapBuffer(NN_GX_DISPLAY0);

		dbgTimerWnd.TimerEnd(3);

		// ======================================================================
		// Bottom display
		// ======================================================================

		// ----------------------------------------------------------------------
		// Generate render texture
		// ----------------------------------------------------------------------
		u32 bmpWidth		= proc_width;
		u32 bmpHeight		= proc_height;
		u32 textureWidth	= 0;
		u32 textureHeight	= 0;

		dbgTimerWnd.TimerStart(4);
		int quality = menuWnd.GetDrawQuality();
		u8* texBuffer = graphics.GetTextureDataFromBmpFileData(bmpWidth, bmpHeight, rgbBuffer, textureWidth, textureHeight, quality);
		dbgTimerWnd.TimerEnd(4);

		// ----------------------------------------------------------------------
		// Rendering
		// ----------------------------------------------------------------------
		dbgTimerWnd.TimerStart(5);
		graphics.ClearDisplay(NN_GX_DISPLAY1);
		graphics.DrawCamera(texBuffer, textureWidth, textureHeight, bmpWidth, bmpHeight, quality);
		dbgTimerWnd.TimerEnd(5);

		graphics.DrawDateTime();
		dbgTimerWnd.TimerStart(6);

		// ----------------------------------------------------------------------
		// Render recognition results
		// ----------------------------------------------------------------------
		fpoint	= hgResult.fpoint ;
		fdir	= hgResult.fdir;
		for (n = 0 ; n < fingernum ; n++ )
		{
			// Start point
			float sx	= (float)fpoint->x * ((float)view_width  / (float)proc_width);
			float sy	= (float)fpoint->y * ((float)view_height / (float)proc_height);

			// Direction
			float dir	= (short) * fdir + 180.0f;
			dir -= (dir > 360.0f) ? 360.0f : 0.0f;
			dir += (dir < 0.0f  ) ? 360.0f : 0.0f;

			// End point
			float ex	= sx + 30.0f * nn::math::CosFIdx(dir / 360.0f * 256.0f);
			float ey	= sy - 30.0f * nn::math::SinFIdx(dir / 360.0f * 256.0f);

			// Arrow
			float ldir = dir + 135.0f;
			float rdir = dir - 135.0f;

			ldir -= (ldir > 360.0f) ? 360.0f : 0.0f;
			ldir += (ldir < 0.0f  ) ? 360.0f : 0.0f;

			rdir -= (rdir > 360.0f) ? 360.0f : 0.0f;
			rdir += (rdir < 0.0f  ) ? 360.0f : 0.0f;

			float alx	= ex + 10.0f * nn::math::CosFIdx(ldir / 360.0f * 256.0f);
			float aly	= ey - 10.0f * nn::math::SinFIdx(ldir / 360.0f * 256.0f);
			float arx	= ex + 10.0f * nn::math::CosFIdx(rdir / 360.0f * 256.0f);
			float ary	= ey - 10.0f * nn::math::SinFIdx(rdir / 360.0f * 256.0f);

			graphics.SetColor(NN_GX_DISPLAY1, 1.0, 0.0, 0.0);
			graphics.DrawLine(NN_GX_DISPLAY1, sx, sy, ex, ey);
			graphics.DrawLine(NN_GX_DISPLAY1, ex, ey, alx, aly);
			graphics.DrawLine(NN_GX_DISPLAY1, ex, ey, arx, ary);
			fpoint++;
			fdir++;
		}

		// ----------------------------------------------------------------------
		// Menu operation through controller operation
		// ----------------------------------------------------------------------
		// Select Window with LR Buttons
		// Operate items with UP/DOWN Buttons
		// Modify values with AB Buttons
		// Output to SD with X Button
		// Exit with Y Button
		// ----------------------------------------------------------------------
		padReader.ReadLatest(&padStatus);

		if (txtWinSort == 0)
		{
			menuWnd.Ctrl(&padStatus);
		}

		// L Button Determination (minus)
		if (padStatus.trigger & BUTTON_L)
		{
			txtWinSort -= (txtWinSort > 0) ? 1 : -2;
		}

		// R Button Determination (plus)
		if (padStatus.trigger & BUTTON_R)
		{
			txtWinSort += (txtWinSort < 2) ? 1 : -2;
		}

		// Switch focus
		menuWnd.SetForcus(txtWinSort == 0);
		resultWnd.SetForcus(txtWinSort == 1);
		dbgTimerWnd.SetForcus(txtWinSort == 2);

		// Output to SD with the X Button
		if (padStatus.trigger & BUTTON_X)
		{
			if (dbg_init == false)
			{
				dbg_init = dbgWriter.Initialize();
			}

			// Outputs only when initialization succeeds
			if (dbg_init == true)
			{
				dbgWriter.Output(rgbBuffer, proc_width, proc_height, &hgResult);
			}
		}

		// Quit with Y Button
		if (padStatus.trigger & BUTTON_Y)
		{
			break;
		}

		dbgTimerWnd.TimerEnd(6);

		// ----------------------------------------------------------------------
		// Swap
		// ----------------------------------------------------------------------
		dbgTimerWnd.TimerStart(7);
		graphics.SwapBuffer(NN_GX_DISPLAY1);
		dbgTimerWnd.TimerEnd(7);

		// ----------------------------------------------------------------------
		// wait vsync
		// ----------------------------------------------------------------------
		graphics.WaitVsync();
		dbgTimerWnd.TimerAllEnd();
	}

	///////////////////////////
	///////////////////////////
	///////////////////////////
	// Free all memory areas.
	///////////////////////////
	///////////////////////////
	///////////////////////////
    OS_FREE( rgbBuffer );
    OS_FREE( yuvBuffer );

	if (extBuf != NULL)
	{
		OS_FREE(extBuf);
		extBuf = NULL;
	}

	graphics.FinalizeGraphics();
	camera.FinalizeCamera();

	// Destroy hid
	nn::hid::Finalize();

	FILE_IO_TERM();
	OS_MEM_TERM();

	//////////////////
	//////////////////
	//////////////////
	// Return.
	//////////////////
	//////////////////
	//////////////////
	PRINT_OUT("Program END\n");

	return ;
}

//	End of File ////////////////////////////////////////////////////////////////////////
